############# check required packages and install accordingly ###################
message('Checking packages.. Please wait!')
#if (!require("data.table"))
#{
	#remove.packages("data.table")
	#install.packages("data.table", type = "source", repos = "https://Rdatatable.gitlab.io/data.table")
#}
#if (!require("R.utils")) install.packages("R.utils", repos="http://cran.us.r-project.org")
#if (!require("dplyr")) install.packages("dplyr", repos="http://cran.us.r-project.org")
#if (!require("Seurat")) install.packages("Seurat", repos="http://cran.us.r-project.org")
#if (!require("rmarkdown")) install.packages("rmarkdown", repos="http://cran.us.r-project.org", dep = T)
#if (!require("kableExtra")) install.packages("kableExtra", repos="http://cran.us.r-project.org")
#if (!require("optparse")) install.packages("optparse", repos="http://cran.us.r-project.org")
#if (!require("Matrix")) install.packages("Matrix", repos="http://cran.us.r-project.org")
#if (!require("remotes")) install.packages("remotes", repos="http://cran.us.r-project.org")
if (!require("presto")) remotes::install_github("immunogenomics/presto")
#if (!require("ggplot2")) install.packages("ggplot2", repos="http://cran.us.r-project.org")
#if (!require("DT")) install.packages("DT", repos="http://cran.us.r-project.org")
##################################################################################

########## Load Libraries ############################
message('Loading Libraries..')
library(R.utils)
library(dplyr)
library(data.table)
library(Seurat)
library(rmarkdown)
library(kableExtra)
library(optparse)
library(Matrix)
library(presto)
library(ggplot2)
library(DT)
options(Seurat.object.assay.version = "v5")
######################################################


################# Set Arguments to be passed ############
message('Reaading Users arguments..')
option_list = list(
	make_option("--pathtoCogentDSreporter", type="character", default=NULL, help="Path to CogentDS reporter script", metavar="character"),
	make_option("--analyzeStats", type="character", default=NULL, help="Analyze Stats file generated by CogentAP", metavar="character"),
  make_option("--geneCounts", type="character", default=NULL, help="Gene counts across cells", metavar="character"),
  make_option("--isoformCounts", type="character", default=NULL, help="Isoform counts across cells", metavar="character"),
  make_option("--geneInfo", type="character", default=NULL, help="Gene Metadata", metavar="character"),
  make_option("--isoformInfo", type="character", default=NULL, help="Isoform Metadata", metavar="character"),
  make_option("--clonotypeCounts", type="character", default=NULL, help="Immune Clonotype Counts", metavar="character"),
  make_option("--clonotypeSummary", type="character", default=NULL, help="Immune Clonotype Summary File", metavar="character"),
  make_option("--clonotypeMetadata", type="character", default=NULL, help="Immune Clonotype Metadata", metavar="character"),
  make_option("--fusionjunctionCounts", type="character", default=NULL, help="Fusion Junction Counts", metavar="character"),
  make_option("--fusionspanFragments", type="character", default=NULL, help="Fusion Span Fragment Counts", metavar="character"),
  make_option("--outPath", type="character", default=NULL, help="Path to write Output files", metavar="character"),
  make_option("--outSeuratObject", type="character", default='CogentDS_analysis.rds', help="Output Seurat v5 Object", metavar="character"),
  make_option("--outSeuratObject_Isoform", type="character", default='CogentDS_isoform_only_analysis.rds', help="Output Seurat v5 Object", metavar="character"),
  make_option("--analyzeIsoformforReport", type="character", default=FALSE, help="is either TRUE / FALSE, by default: FALSE", metavar="character"),
  make_option("--outHTMLReport", type="character", default='CogentDS_preliminary-analysis_report.html', help="HTML report of analysis", metavar="character")

);  
opt_parser = OptionParser(option_list=option_list);
opt = parse_args(opt_parser);
#######################################################

########### Create path to required scripts ##########
if(!is.null(opt$pathtoCogentDSreporter)) {
	FastRead_largeCSV_genes = paste0(opt$pathtoCogentDSreporter, "/bin/FastRead_largeCSV_genes.R")
	FastRead_largeCSV_isoforms = paste0(opt$pathtoCogentDSreporter, "/bin/FastRead_largeCSV_isoforms.R")
	Get_ClonotypeData = paste0(opt$pathtoCogentDSreporter, "/bin/Get_ClonotypeData.R")
	Get_FusionData = paste0(opt$pathtoCogentDSreporter, "/bin/Get_FusionData.R")
	Generate_HTML_report = paste0(opt$pathtoCogentDSreporter, "/bin/Generate_HTML_report.R")
	if(!is.null(opt$outPath)) {
		dir.create(opt$outPath)
		temppath = paste0(opt$outPath, "/temp/")
		dir.create(temppath)
	} else {
		stop("--outPath is set to NULL!! value needed for writing files is specified directory")
	}
} else {
	stop("FATAL ERROR: Path to 'CogentDS_DataHandler_and_Reporter.R' is missing. Argument not supplied.. Found NULL", call.=FALSE)
}
########################################################


################# Check required Files ###############
message('Checking necessary files..')
if (is.null(opt$geneCounts)){
  print_help(opt_parser)
  stop("At least one argument must be supplied (geneCounts).n", call.=FALSE)
}
#####################################################

###################### Generate Gene Counts Seurat Object ##################
message('Gene Counts seurat object..')
if(!is.null(opt$geneCounts) && !is.null(opt$geneInfo)) {
	source(FastRead_largeCSV_genes, local = TRUE)
} else {
	stop("Gene Counts and Gene Info not found or set to NULL", call.=FALSE)
}
###########################################################################

##################### Generate Isoform Counts Seurat Object ###############
message('Isoform Counts seurat object..')
if(!is.null(opt$isoformCounts) && !is.null(opt$isoformInfo) && !is.null(seuratObject_gene)) {
	source(FastRead_largeCSV_isoforms, local = TRUE)
} else {
	stop("Isoform Counts, Info or Gene counts based Seurat Object missing or set to NULL", call.=FALSE)
}
###########################################################################

################# Generate Clonotype Seurat Object ###################
message('Gathering Clonotype data..')
if(!is.null(opt$clonotypeCounts) && !is.null(opt$clonotypeSummary) && !is.null(opt$clonotypeMetadata) && !is.null(seuratObject_isoform) && !is.null(seuratObject_gene)) {
	source(Get_ClonotypeData, local = TRUE)
} else {
	BCRl_mat = NULL
	BCRh_mat = NULL
	TCRa_mat = NULL
	TCRb_mat = NULL
	TCRdg_mat = NULL
	message("Clonotype metadata, counts or summary is missing or set to NULL")
}
######################################################################

################# Generate Fusion Matrices ###################
message('Gathering Gene Fusion data..')
if(!is.null(opt$fusionjunctionCounts) && !is.null(opt$fusionspanFragments) && !is.null(seuratObject_isoform) && !is.null(seuratObject_gene)) {
	source(Get_FusionData, local = TRUE)
} else {
	junction_reads = NULL
	span_frags = NULL
	message("Fusion counts or span fragments data is missing or set to NULL")
}
######################################################################

############# Generate Final Seurat v5 Object for DS ##################
message('Generating Final Object..')
if(!is.null(seuratObject_isoform) && !is.null(seuratObject_gene)) {
	seuratDSobject = CreateSeuratObject(seuratObject_gene[['RNA']]$counts)
	isoform_counts = seuratObject_isoform[['RNA']]$counts
	isoform_counts = isoform_counts[,colnames(seuratObject_gene[['RNA']]$counts)]
	seuratDSobject[['Transcript_RNA']] = CreateAssayObject(counts = isoform_counts)

	if((nrow(BCRl_mat) > 0) && (!is.null(BCRl_mat))) {
		BCRl_mat = BCRl_mat[,intersect(colnames(seuratObject_gene[['RNA']]$counts), colnames(BCRl_mat))]
		seuratDSobject[['BCRl']] = CreateAssayObject(counts = BCRl_mat)
	}
	if((nrow(BCRh_mat) > 0) && (!is.null(BCRh_mat))) {
		BCRh_mat = BCRh_mat[,intersect(colnames(seuratObject_gene[['RNA']]$counts), colnames(BCRh_mat))]
		seuratDSobject[['BCRh']] = CreateAssayObject(counts = BCRh_mat)
	}
	if((nrow(TCRa_mat) > 0) && (!is.null(TCRa_mat))) {
		TCRa_mat = TCRa_mat[,intersect(colnames(seuratObject_gene[['RNA']]$counts), colnames(TCRa_mat))]
		seuratDSobject[['TCRa']] = CreateAssayObject(counts = TCRa_mat)
	}
	if((nrow(TCRb_mat) > 0) && (!is.null(TCRb_mat))) {
		TCRb_mat = TCRb_mat[,intersect(colnames(seuratObject_gene[['RNA']]$counts), colnames(TCRb_mat))]
		seuratDSobject[['TCRb']] = CreateAssayObject(counts = TCRb_mat)
	}
	if((nrow(TCRdg_mat) > 0) && (!is.null(TCRdg_mat))) {
		TCRdg_mat = TCRdg_mat[,intersect(colnames(seuratObject_gene[['RNA']]$counts), colnames(TCRdg_mat))]
		seuratDSobject[['TCRdg']] = CreateAssayObject(counts = TCRdg_mat)
	}

	if((nrow(junction_reads) > 0) && (!is.null(junction_reads))) {
		junction_reads = junction_reads[,intersect(colnames(seuratObject_gene[['RNA']]$counts), colnames(junction_reads))]
		seuratDSobject[['junction_reads']] = CreateAssayObject(counts = junction_reads)
	}

	if((nrow(span_frags) > 0) && (!is.null(span_frags))) {
		span_frags = span_frags[,intersect(colnames(seuratObject_gene[['RNA']]$counts), colnames(span_frags))]
		seuratDSobject[['span_frags']] = CreateAssayObject(counts = span_frags)
	}

	if(!is.null(opt$analyzeStats)) {
		stats = read.csv(opt$analyzeStats, header = TRUE, check.names = FALSE)
		if(length(unique(stats$Barcode)) > 1) {
			rownames(stats) = stats$Barcode
		} else {
			rownames(stats) = stats$Sample
		}
		DefaultAssay(seuratDSobject) = "RNA"
		stats = stats[colnames(seuratDSobject),]
		seuratDSobject = AddMetaData(seuratDSobject, metadata = stats)
	} else {
		message("analyze stats either set to NULL or not provided")
	}

	outfile = paste0(opt$outPath, "/", opt$outSeuratObject)
	saveRDS(seuratDSobject, file = outfile)
} else {
	stop("Gene and Isoform Count based Seurat Object missing or set to Null")
}
#########################################################################

################ Generate DS Report from Seurat Object ##################
message('Generating Report..')
if(!is.null(seuratDSobject))
{
	seuratDSobject_file = paste0(opt$outPath, "/", opt$outSeuratObject)
	source(Generate_HTML_report, local = TRUE)
	file_path = paste0(opt$outPath, "/", opt$outHTMLReport)
	if (file.exists(file_path)) {
		message('Seurat v5 object created and report generated')
	} else {
		message('Report generation failed')
	}
	unlink(temppath, recursive = TRUE)
} else {
	stop('Seurat v5 Object generation has failed! Please check inputs')
}
