library(dplyr)

immune_summary = read.csv(opt$clonotypeSummary, header = T, row.names = 1, check.names = F)
immune_meta = read.csv(opt$clonotypeMetadata, header = T, row.names = 1)
immune_matrix = read.csv(opt$clonotypeCounts, header = T, row.names = 1, check.names = F)

immune_meta_light = immune_meta %>% dplyr::filter(Light.Chain == "Y")
immune_meta_heavy = immune_meta %>% dplyr::filter(Heavy.Chain == "Y")

cols_immune_summary = colnames(immune_summary)

cols_immune_summary_imp = grep("_", cols_immune_summary, value = T)

not_needed_cols = 'Total_Reads'

cols_immune_summary_imp = cols_immune_summary_imp[! cols_immune_summary_imp %in% not_needed_cols]

cols_immune_summary_imp_TRA = grep('TRA', cols_immune_summary_imp, value = T)

cols_immune_summary_imp_TRB = grep('TRB', cols_immune_summary_imp, value = T)

cols_immune_summary_imp_TRD = grep('TRD', cols_immune_summary_imp, value = T)

cols_immune_summary_imp_TRG = grep('TRG', cols_immune_summary_imp, value = T)

cols_immune_summary_imp_TRDG = c(cols_immune_summary_imp_TRD, cols_immune_summary_imp_TRG)

TCR = c(cols_immune_summary_imp_TRA, cols_immune_summary_imp_TRB, cols_immune_summary_imp_TRDG)

cols_immune_summary_imp_B = cols_immune_summary_imp[! cols_immune_summary_imp %in% TCR]

TRA_values = c()
for(f in 1:length(cols_immune_summary_imp_TRA))
{
	immune_summary_temp = immune_summary %>% dplyr::select(cols_immune_summary_imp_TRA[f])
	values = unique(immune_summary_temp[[cols_immune_summary_imp_TRA[f]]])
	values = values[! values %in% c(".", "")]
	TRA_values = c(TRA_values, values)
}
TRA_values <- TRA_values[!is.na(TRA_values)]


TRB_values = c()
for(f in 1:length(cols_immune_summary_imp_TRB))
{
	immune_summary_temp = immune_summary %>% dplyr::select(cols_immune_summary_imp_TRB[f])
	values = unique(immune_summary_temp[[cols_immune_summary_imp_TRB[f]]])
	values = values[! values %in% c(".", "")]
	TRB_values = c(TRB_values, values)
}
TRB_values <- TRB_values[!is.na(TRB_values)]

TRDG_values = c()
for(f in 1:length(cols_immune_summary_imp_TRDG))
{
	immune_summary_temp = immune_summary %>% dplyr::select(cols_immune_summary_imp_TRDG[f])
	values = unique(immune_summary_temp[[cols_immune_summary_imp_TRDG[f]]])
	values = values[! values %in% c(".", "")]
	TRDG_values = c(TRDG_values, values)
}
TRDG_values <- TRDG_values[!is.na(TRDG_values)]
#TRDG_values_notneeded1 = grep("TRA", TRDG_values, value = T)
#TRDG_values_notneeded2 = grep("TRB", TRDG_values, value = T)
#trdg_notneeded = c(TRDG_values_notneeded1, TRDG_values_notneeded2)
#TRDG_values = TRDG_values[! TRDG_values %in% trdg_notneeded]

IG_values = c()
for(f in 1:length(cols_immune_summary_imp_B))
{
	immune_summary_temp = immune_summary %>% dplyr::select(cols_immune_summary_imp_B[f])
	values = unique(immune_summary_temp[[cols_immune_summary_imp_B[f]]])
	values = values[! values %in% c(".", "")]
	IG_values = c(IG_values, values)
}
IG_values <- IG_values[!is.na(IG_values)]

if(length(TRA_values) > 0) {
	TRA_values_split = strsplit(TRA_values, "*", fixed = T)
	TRA_values = c()
	for(f in 1:length(TRA_values_split))
	{
		values = TRA_values_split[[f]][1]
		TRA_values = c(TRA_values, values)
	}
	TRA_rownames = c()
	for(f in 1:length(TRA_values))
	{
		get_val = grep(as.character(TRA_values[f]), rownames(immune_matrix), value = T)
		if(!is.null(get_val) == TRUE)
		{
			TRA_rownames = c(TRA_rownames, get_val)
		}
	}
	TRA_rownames = unique(TRA_rownames)
} else {
	TRA_values = NULL
	TRA_rownames = NULL
}


if(length(TRB_values) > 0) {
	TRB_values_split = strsplit(TRB_values, "*", fixed = T)
	TRB_values = c()
	for(f in 1:length(TRB_values_split))
	{
		values = TRB_values_split[[f]][1]
		TRB_values = c(TRB_values, values)
	}

	TRB_rownames = c()
	for(f in 1:length(TRB_values))
	{
		get_val = grep(as.character(TRB_values[f]), rownames(immune_matrix), value = T)
		if(!is.null(get_val) == TRUE)
		{
			TRB_rownames = c(TRB_rownames, get_val)
		}
	}
	TRB_rownames = unique(TRB_rownames)
} else {
	TRB_values = NULL
	TRB_rownames = NULL
}

if(length(TRDG_values) > 0) {
	TRDG_values_split = strsplit(TRDG_values, "*", fixed = T)
	TRDG_values = c()
	for(f in 1:length(TRDG_values_split))
	{
		values = TRDG_values_split[[f]][1]
		TRDG_values = c(TRDG_values, values)
	}

	TRDG_rownames = c()
	for(f in 1:length(TRDG_values))
	{
		get_val = grep(as.character(TRDG_values[f]), rownames(immune_matrix), value = T)
		if(!is.null(get_val) == TRUE)
		{
			TRDG_rownames = c(TRDG_rownames, get_val)
		}
	}
	TRDG_rownames = unique(TRDG_rownames)
} else {
	TRDG_values = NULL
	TRDG_rownames = NULL
}

if(length(IG_values) > 0) {
	IG_values_split = strsplit(IG_values, "*", fixed = T)
	IG_values = c()
	for(f in 1:length(IG_values_split))
	{
		values = IG_values_split[[f]][1]
		IG_values = c(IG_values, values)
	}

	IG_rownames = c()
	for(f in 1:length(IG_values))
	{
		get_val = grep(as.character(IG_values[f]), rownames(immune_matrix), value = T)
		if(!is.null(get_val) == TRUE)
		{
			IG_rownames = c(IG_rownames, get_val)
		}
	}
	IG_rownames = unique(IG_rownames)
} else {
	IG_values = NULL
	IG_rownames = NULL
}


found = c(TRA_rownames, TRB_rownames, TRDG_rownames, IG_rownames)
notfound = rownames(immune_matrix)[! rownames(immune_matrix) %in% found]

common_TRA_TRDG = intersect(TRA_rownames, TRDG_rownames)
unique_TRA = setdiff(TRA_rownames, TRDG_rownames)
unique_TRDG = setdiff(TRDG_rownames, TRA_rownames)

TRA_rownames = c()
TRA_rownames = c(common_TRA_TRDG, unique_TRA)
TRDG_rownames = c()
TRDG_rownames = unique_TRDG

if(length(notfound) == 0)
{
	IG_rownames_light = intersect(IG_rownames, rownames(immune_meta_light))
	IG_rownames_heavy = intersect(IG_rownames, rownames(immune_meta_heavy))

	check_clonotypes = length(IG_rownames_heavy) + length(IG_rownames_light) + length(TRB_rownames) + length(TRA_rownames) + length(TRDG_rownames)
	if(check_clonotypes == length(rownames(immune_matrix)))
	{
		BCRh_mat = immune_matrix[IG_rownames_heavy,]
		BCRl_mat = immune_matrix[IG_rownames_light,]
		TCRb_mat = immune_matrix[TRB_rownames,]
		TCRa_mat = immune_matrix[TRA_rownames,]
		TCRdg_mat = immune_matrix[TRDG_rownames,]
	} else {
		BCRh_mat = NULL
		BCRl_mat = NULL
		TCRb_mat = NULL
		TCRa_mat = NULL
		TCRdg_mat = NULL
	}
} else {
	BCRh_mat = NULL
	BCRl_mat = NULL
	TCRb_mat = NULL
	TCRa_mat = NULL
	TCRdg_mat = NULL
}
